# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("aimmo", "0005_auto_20160808_1545")]

    operations = [
        migrations.AddField(
            model_name="game",
            name="static_data",
            field=models.TextField(null=True, blank=True),
        )
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.24 on 2020-04-15 11:38
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("aimmo", "0012_auto_20200302_1310")]

    operations = [
        migrations.AlterField(
            model_name="game",
            name="can_play",
            field=models.ManyToManyField(
                help_text="List of auth_user IDs of users who are allowed to play and have access to the game.",
                related_name="playable_games",
                to=settings.AUTH_USER_MODEL,
            ),
        )
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.24 on 2020-07-23 11:43
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("common", "0001_initial"),
        ("portal", "0058_move_to_common_models"),
        ("aimmo", "0015_game_worksheet"),
    ]

    operations = [
        migrations.AddField(
            model_name="game",
            name="game_class",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="games_for_class",
                to="common.Class",
            ),
        ),
    ]



#!/usr/bin/env python

import asyncio
import json
import logging
import os
from dataclasses import dataclass
from typing import Any, Awaitable, Dict
from urllib.parse import parse_qs

import aiohttp_cors
import google.cloud.logging
import grpc
import socketio
from aiohttp import web
from google.auth.exceptions import DefaultCredentialsError
from kubernetes.config import load_incluster_config
from socketio.asyncio_server import AsyncServer

from activity_monitor import ActivityMonitor
from agones import sdk_pb2
from agones.sdk_pb2_grpc import SDKStub as AgonesSDKStub
from authentication import initialize_game_token
from simulation import map_generator
from simulation.django_communicator import DjangoCommunicator
from simulation.game_runner import GameRunner
from simulation.log_collector import LogCollector
from turn_collector import TurnCollector

LOGGER = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO)


def setup_application(communicator: DjangoCommunicator, should_clean_token=True):
    async def clean_token(app):
        LOGGER.info("Cleaning token!")
        await communicator.patch_token(data={"token": ""})

    application = web.Application()

    if should_clean_token:
        application.on_shutdown.append(clean_token)

    application.on_shutdown.append(communicator.close_session)

    return application


def setup_socketIO_server(application, async_handlers=True):
    socket_server = AsyncServer(
        async_mode="aiohttp",
        client_manager=socketio.AsyncManager(),
        async_handlers=async_handlers,
        cors_allowed_origins=[
            "http://localhost:8000",
            "https://dev-dot-decent-digit-629.appspot.com",
            "https://staging-dot-decent-digit-629.appspot.com",
            "https://www.codeforlife.education",
        ],
    )

    socket_server.attach(
        application, socketio_path=os.environ.get("SOCKETIO_RESOURCE", "socket.io")
    )

    return socket_server


class GameAPI(object):
    def __init__(
        self,
        game_state,
        application,
        socketio_server: AsyncServer,
        activity_monitor: ActivityMonitor,
    ):
        self.app = application
        self.socketio_server = socketio_server
        self.register_endpoints()
        self.game_state = game_state
        self.log_collector = LogCollector(game_state.avatar_manager)
        self.activity_monitor = activity_monitor

    async def async_map(self, func, iterable_args):
        futures = [func(arg) for arg in iterable_args]
        await asyncio.gather(*futures)

    def register_endpoints(self):
        self.routes = web.RouteTableDef()
        self.register_healthcheck()
        self.register_world_update_on_connect()
        self.register_remove_session_id_from_mappings()
        self.app.add_routes(self.routes)

    def open_connections_number(self):
        try:
            return len(
                [x for x in self.socketio_server.manager.get_participants("/", None)]
            )
        except KeyError:
            return 0

    def update_active_users(self):
        self.activity_monitor.active_users = self.open_connections_number()

    def register_healthcheck(self):
        @self.routes.get("/game-{game_id}")
        async def healthcheck(request):
            return web.Response(text="HEALTHY")

        return healthcheck

    def register_world_update_on_connect(self):
        @self.socketio_server.on("connect")
        async def world_update_on_connect(sid, environ):
            query = environ["QUERY_STRING"]
            avatar_id = self._find_avatar_id_from_query(sid, query)
            await self.socketio_server.save_session(sid, {"id": avatar_id})
            LOGGER.info(f"Socket connected for session id: {sid}")

        return world_update_on_connect

    def register_remove_session_id_from_mappings(self):
        @self.socketio_server.on("disconnect")
        async def remove_session_id_from_mappings(sid):
            LOGGER.info("Socket disconnected for session id: {}. ".format(sid))

        return remove_session_id_from_mappings

    async def send_updates(self, sid):
        try:
            LOGGER.info(f"sending updates to {sid}")
            await self._send_game_state(sid)
        except KeyError as e:
            LOGGER.error(f"Failed to send updates: {e}")

    async def send_updates_to_all(self):
        try:
            socket_ids = [
                sid
                for (sid, _) in self.socketio_server.manager.get_participants("/", None)
            ]
            LOGGER.info(f"socket_ids: f{socket_ids}")
            await self.async_map(self.send_updates, socket_ids)
        except KeyError as e:
            LOGGER.warning("No open socket connections")
            LOGGER.error(e)
        self.update_active_users()

    def _find_avatar_id_from_query(self, session_id, query_string):
        """
        :param session_id: Int with the session id
        :param query_string: String from the environment settings,
        usually located as the key 'QUERY_STRING'.
        """
        parsed_qs = parse_qs(query_string)

        try:
            avatar_id = int(parsed_qs["avatar_id"][0])
            return avatar_id
        except ValueError:
            LOGGER.error("Avatar ID could not be casted into an integer")
        except KeyError:
            LOGGER.error("No avatar ID found. User may not be authorised")
            LOGGER.error(f"query_string: {query_string}")

    async def _send_game_state(self, sid):
        session_data = await self.socketio_server.get_session(sid)
        serialized_game_state = self.game_state.serialize()
        serialized_game_state["playerLog"] = self.log_collector.collect_logs(
            session_data["id"]
        )
        await self.socketio_server.emit("game-state", serialized_game_state, room=sid)


def create_runner(port, socketio_server, communicator: DjangoCommunicator):
    settings = json.loads(os.environ["settings"])
    generator = getattr(map_generator, settings["GENERATOR"])(settings)
    turn_collector = TurnCollector(socketio_server)
    return GameRunner(
        game_state_generator=generator.get_game_state,
        communicator=communicator,
        port=port,
        turn_collector=turn_collector,
    )


future_app = asyncio.Future()
future_port = asyncio.Future()


def run_game(port, game_id, django_api_url):
    global cors

    communicator = DjangoCommunicator(django_api_url=django_api_url)
    activity_monitor = ActivityMonitor(communicator)

    app = setup_application(communicator)
    cors = aiohttp_cors.setup(app)
    socketio_server = setup_socketIO_server(app)

    game_runner = create_runner(port, socketio_server, communicator)

    asyncio.ensure_future(initialize_game_token(communicator, game_id))

    game_api = GameAPI(
        game_state=game_runner.game_state,
        application=app,
        socketio_server=socketio_server,
        activity_monitor=activity_monitor,
    )
    game_runner.set_end_turn_callback(game_api.send_updates_to_all)

    asyncio.ensure_future(game_runner.run())
    future_app.set_result(app)


@dataclass
class GameAllocationInfo:
    game_id: int
    django_api_url: str
    port: int = 5000


async def wait_for_allocation(
    agones_stub: AgonesSDKStub,
) -> Awaitable[GameAllocationInfo]:
    empty_request = sdk_pb2.Empty()
    async for game_server_update in agones_stub.WatchGameServer(empty_request):
        if game_server_update.status.state == "Allocated":
            LOGGER.info(f"Game server allocated")
            labels: Dict[str, Any] = game_server_update.object_meta.labels
            annotations: Dict[str, Any] = game_server_update.object_meta.annotations
            game_id = labels["game-id"]
            os.environ["worksheet_id"] = annotations["worksheet_id"]
            os.environ["GAME_API_URL"] = annotations["GAME_API_URL"]
            os.environ["settings"] = annotations["settings"]
            django_api_url = annotations["GAME_API_URL"]
            return GameAllocationInfo(game_id, django_api_url)


def setup_healthcheck(agones_stub: AgonesSDKStub):
    def empty_response_generator():
        while True:
            emp_request = sdk_pb2.Empty()
            yield emp_request

    agones_stub.Health(empty_response_generator())


def send_ready_state(agones_stub: AgonesSDKStub):
    empty_request = sdk_pb2.Empty()
    agones_stub.Ready(empty_request)
    LOGGER.info("Game server ready for allocation")


async def get_app():
    app = await future_app
    return app


def setup_logging():
    logging.basicConfig(level=logging.DEBUG)

    logging.getLogger("socketio").setLevel(logging.ERROR)
    logging.getLogger("engineio").setLevel(logging.ERROR)
    logging.getLogger("aiohttp.server").setLevel(logging.INFO)

    try:
        logging_client = google.cloud.logging.Client()
        logging_client.get_default_handler()
        logging_client.setup_logging()
    except DefaultCredentialsError:
        logging.info(
            "No google credentials provided, not connecting google logging client"
        )


if __name__ == "__main__":
    event_loop = asyncio.get_event_loop()
    channel = grpc.aio.insecure_channel(
        f"localhost:{os.environ['AGONES_SDK_GRPC_PORT']}",
        options=(("grpc.enable_http_proxy", 0),),
    )
    agones_stub = AgonesSDKStub(channel)
    load_incluster_config()
    setup_logging()
    setup_healthcheck(agones_stub)
    send_ready_state(agones_stub)
    game_metadata: GameAllocationInfo = event_loop.run_until_complete(
        wait_for_allocation(agones_stub)
    )

    run_game(game_metadata.port, game_metadata.game_id, game_metadata.django_api_url)
    web.run_app(get_app(), port=game_metadata.port)



from django.contrib.auth.models import User
from django.core.management import BaseCommand


class Command(BaseCommand):
    # Show this when the user types help
    help = "Delete generated users"

    # A command must define handle()
    def handle(self, *args, **options):
        for user in User.objects.filter(username__startswith="zombie-"):
            self.stdout.write("Deleting %s" % user.get_username())
            user.delete()




from aimmo.models import Avatar, Game


def create_avatar_for_user(user, game_id):
    """
    Creates an Avatar object for a user. Sets the initial code to simple avatar code
    (unless specified otherwise).

    :param user: The user the Avatar is for.
    :param game_id: The id of the game in which the Avatar is created.
    :return: The initialised Avatar object.
    """
    game: Game = Game.objects.get(id=game_id)
    initial_code = game.worksheet.starter_code
    avatar = Avatar.objects.create(owner=user, code=initial_code, game_id=game_id)
    return avatar



# This will probably replace the whole aimmo-game-creator at some point.
# For now it just duplicates a part of aimmo-game-creator/game_manager.py in order to recreate a game server.
import logging
import time

import kubernetes
from kubernetes.client import CoreV1Api
from kubernetes.client.api.custom_objects_api import CustomObjectsApi
from kubernetes.client.api_client import ApiClient
from kubernetes.client.rest import ApiException

LOGGER = logging.getLogger(__name__)

K8S_NAMESPACE = "default"
AGONES_GROUP = "agones.dev"


class GameManager:
    def __init__(self) -> None:
        self.api: CoreV1Api = CoreV1Api()
        self.api_client: ApiClient = ApiClient()
        self.custom_objects_api: CustomObjectsApi = CustomObjectsApi(self.api_client)

    @staticmethod
    def create_game_name(game_id: int) -> str:
        """
        Creates a name that will be used as the pod name as well as in other places.

        :param game_id: Integer indicating the GAME_ID of the game.
        :return: A string with the game appended with the id.
        """
        return f"game-{game_id}"

    def patch_game_service(self, game_id, game_server_name):
        patched_service = kubernetes.client.V1Service(
            spec=kubernetes.client.V1ServiceSpec(
                selector={"agones.dev/gameserver": game_server_name}
            )
        )
        self.api.patch_namespaced_service(
            self.create_game_name(game_id), K8S_NAMESPACE, patched_service
        )

    def create_game_server_allocation(
        self, game_id: int, game_data: dict, retry_count: int = 0
    ) -> str:
        result = self.custom_objects_api.create_namespaced_custom_object(
            group="allocation.agones.dev",
            version="v1",
            namespace=K8S_NAMESPACE,
            plural="gameserverallocations",
            body={
                "apiVersion": "allocation.agones.dev/v1",
                "kind": "GameServerAllocation",
                "metadata": {"generateName": "game-allocation-"},
                "spec": {
                    "required": {"matchLabels": {"agones.dev/fleet": "aimmo-game"}},
                    "scheduling": "Packed",
                    "metadata": {
                        "labels": {
                            "game-id": str(game_id),
                        },
                        "annotations": game_data,
                    },
                },
            },
        )
        if result["status"]["state"] == "UnAllocated" and retry_count < 60:
            LOGGER.warning(
                f"Failed to create game, retrying... retry_count={retry_count}"
            )
            time.sleep(5)
            return self.create_game_server_allocation(
                game_id, game_data, retry_count=retry_count + 1
            )
        else:
            return result["status"]["gameServerName"]

    def delete_game_server(self, game_id: int) -> dict:
        """
        Delete the game server with the specified game_id and return its game data.

        :param game_id: Integer indicating the ID of the game to delete.
        :returns: A dictionary representing the game data.
        """
        game_data = {}
        result = self.custom_objects_api.list_namespaced_custom_object(
            group=AGONES_GROUP,
            version="v1",
            namespace=K8S_NAMESPACE,
            plural="gameservers",
            label_selector=f"game-id={game_id}",
        )
        game_servers_to_delete = result["items"]

        if len(game_servers_to_delete) == 0:
            LOGGER.warning(
                f"delete_game_server - No game server found with ID {game_id}"
            )
        elif len(game_servers_to_delete) > 1:
            LOGGER.warning(
                f"delete_game_server - Multiple game servers found with ID {game_id}"
            )

        for game_server in game_servers_to_delete:
            name = game_server["metadata"]["name"]
            game_data.update(game_server["metadata"]["annotations"])
            self.custom_objects_api.delete_namespaced_custom_object(
                group=AGONES_GROUP,
                version="v1",
                namespace=K8S_NAMESPACE,
                plural="gameservers",
                name=name,
            )

        # Remove agones specific annotations from game_data
        game_data = {
            k: v for k, v in game_data.items() if not k.startswith(f"{AGONES_GROUP}/")
        }
        return game_data

    def recreate_game_server(
        self, game_id: int, game_data_updates: dict = None
    ) -> None:
        """
        Recreate a game server with the specified game_id and optionally update its game data.

        :param game_id: Integer indicating the ID of the game to recreate.
        :param game_data_updates: Optional, a dictionary with game data updates.
        :returns: None
        """
        if game_data_updates is None:
            game_data_updates = {}

        game_data = self.delete_game_server(game_id=game_id)
        game_data.update(game_data_updates)
        game_server_name = self.create_game_server_allocation(
            game_id=game_id, game_data=game_data
        )
        self.patch_game_service(game_id=game_id, game_server_name=game_server_name)

    def create_game_secret(self, game_id, token):
        name = self.create_game_name(game_id) + "-token"
        body = kubernetes.client.V1Secret(
            kind="Secret",
            string_data={"token": token},
            metadata=kubernetes.client.V1ObjectMeta(
                name=name,
                namespace=K8S_NAMESPACE,
                labels={"game_id": str(game_id), "app": "aimmo-game"},
            ),
        )
        try:
            self.api.read_namespaced_secret(name, K8S_NAMESPACE)
        except ApiException:
            try:
                self.api.create_namespaced_secret(namespace=K8S_NAMESPACE, body=body)
            except ApiException:
                LOGGER.exception("Exception when calling create_namespaced_secret")
        else:
            try:
                self.api.patch_namespaced_secret(
                    name=name, namespace=K8S_NAMESPACE, body=body
                )
            except ApiException:
                LOGGER.exception("Exception when calling patch_namespaced_secret")

    def delete_game_secret(self, game_id):
        app_label = "app=aimmo-game"
        game_label = "game_id={}".format(game_id)

        resources = self.api.list_namespaced_secret(
            namespace=K8S_NAMESPACE, label_selector=",".join([app_label, game_label])
        )

        for resource in resources.items:
            LOGGER.info("Removing game secret: {}".format(resource.metadata.name))
            self.api.delete_namespaced_secret(
                name=resource.metadata.name, namespace=K8S_NAMESPACE
            )



import base64
import logging
import os

from kubernetes.client import CoreV1Api

LOGGER = logging.getLogger(__name__)


def _decode_token_from_secret(secret):
    return base64.b64decode(secret.data["token"]).decode()


async def initialize_game_token(communicator, game_id):
    """Get game token and store it somewhere accessible."""
    api = CoreV1Api()
    secret = api.read_namespaced_secret(f"game-{game_id}-token", "default")
    os.environ["TOKEN"] = _decode_token_from_secret(secret)
    LOGGER.info("Token set!")

    response = await communicator.patch_token({"token": os.environ["TOKEN"]})
    if response.status != 200:
        LOGGER.error(response)



import logging
from typing import Tuple

from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.http import Http404, HttpResponse, HttpResponseForbidden, JsonResponse
from django.shortcuts import get_object_or_404
from django.views.decorators.csrf import csrf_exempt, ensure_csrf_cookie
from django.views.decorators.http import require_http_methods
from rest_framework import mixins, status, viewsets
from rest_framework.authentication import BasicAuthentication, SessionAuthentication
from rest_framework.decorators import (
    api_view,
    authentication_classes,
    permission_classes,
    action,
)
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView

from common.permissions import CanDeleteGame

from . import game_renderer
from .avatar_creator import create_avatar_for_user
from .exceptions import UserCannotPlayGameException
from .models import Avatar, Game
from .permissions import (
    CanDeleteGameOrReadOnly,
    CsrfExemptSessionAuthentication,
    GameHasToken,
)
from .serializers import GameSerializer, GameIdsSerializer

LOGGER = logging.getLogger(__name__)


@login_required
def code(request, id):
    if not request.user:
        print("no user")
        return HttpResponseForbidden()
    game = get_object_or_404(Game, id=id)
    if not game.can_user_play(request.user):
        print("user can't play")
        raise Http404
    try:
        avatar = game.avatar_set.get(owner=request.user)
    except Avatar.DoesNotExist:
        avatar = create_avatar_for_user(request.user, id)
    if request.method == "POST":
        avatar.code = request.POST["code"]
        avatar.save()
        return HttpResponse(status=200)
    else:
        return JsonResponse(
            {"code": avatar.code, "starterCode": game.worksheet.starter_code}
        )


class GameUsersView(APIView):
    authentication_classes = (CsrfExemptSessionAuthentication, BasicAuthentication)
    permission_classes = (GameHasToken,)

    def get(self, request, id):
        game = get_object_or_404(Game, id=id)
        data = self.serialize_users(game)
        return JsonResponse(data)

    def serialize_users(self, game):
        users = {"main_avatar": None, "users": []}
        for avatar in game.avatar_set.all():
            if avatar.owner_id == game.main_user_id:
                users["main_avatar"] = avatar.id
            users["users"].append({"id": avatar.id, "code": avatar.code})
        return users


class GameViewSet(
    viewsets.GenericViewSet,
    mixins.DestroyModelMixin,
    mixins.RetrieveModelMixin,
    mixins.UpdateModelMixin,
):
    authentication_classes = (CsrfExemptSessionAuthentication, SessionAuthentication)
    queryset = Game.objects.all()
    permission_classes = (CanDeleteGameOrReadOnly,)
    serializer_class = GameSerializer

    def list(self, request):
        response = {}
        for game in Game.objects.all():
            serializer = GameSerializer(game)
            response[game.pk] = serializer.data
        return Response(response)

    @action(
        methods=["get"],
        detail=False,
    )
    def running(self, request):
        response = {
            game.pk: GameSerializer(game).data
            for game in Game.objects.filter(status=Game.RUNNING)
        }
        return Response(response)

    @action(
        methods=["post"],
        detail=False,
        serializer_class=GameIdsSerializer,
        permission_classes=(CanDeleteGame,),
    )
    def delete_games(self, request):
        game_ids = request.data.getlist("game_ids")
        Game.objects.filter(
            pk__in=game_ids, game_class__teacher__new_user=request.user
        ).delete()
        return Response(status=status.HTTP_204_NO_CONTENT)


@api_view(["GET"])
@authentication_classes([SessionAuthentication])
@permission_classes([IsAuthenticated])
def connection_parameters(request, game_id):
    """
    An API view which returns the correct connection settings required
    to run the game in different environments. These values will change
    depending on where the project is started (ie. local, etc).
    :param request: Django request object.
    :param game_id: Integer with the ID of the game.
    :return: JsonResponse object with the contents.
    """
    env_connection_settings = game_renderer.get_environment_connection_settings(game_id)

    avatar_id, response = get_avatar_id(request.user, game_id)

    if avatar_id:
        env_connection_settings.update({"avatar_id": avatar_id})
        return JsonResponse(env_connection_settings)
    else:
        return response


class GameTokenView(APIView):
    """
    View to Game tokens, used to prove a request comes from a game.
    """

    authentication_classes = (CsrfExemptSessionAuthentication, BasicAuthentication)
    permission_classes = (GameHasToken,)

    def get(self, request, id):
        """
        After the initial token request, we need to check where the
        request comes from. So for subsequent requests we verify that
        they came from the token-holder.
        """
        game = get_object_or_404(Game, id=id)
        self.check_object_permissions(self.request, game)
        return Response(data={"token": game.auth_token})

    def patch(self, request, id):
        game = get_object_or_404(Game, id=id)
        self.check_object_permissions(self.request, game)
        try:
            game.auth_token = request.data["token"]
            game.save()
            return Response(status=status.HTTP_200_OK)
        except KeyError:
            return Response(status=status.HTTP_403_FORBIDDEN)


@ensure_csrf_cookie
def watch_game(request, id):
    game = get_object_or_404(Game, id=id)
    if not game.can_user_play(request.user):
        raise Http404

    game.status = Game.RUNNING
    game.save()
    return game_renderer.render_game(request, game)


def get_avatar_id(user: User, game_id) -> Tuple[int, HttpResponse]:
    avatar_id = None
    response = Response(status=status.HTTP_200_OK)

    try:
        avatar_id = game_renderer.get_avatar_id_from_user(user=user, game_id=game_id)
    except UserCannotPlayGameException:
        LOGGER.warning(
            "HTTP 401 returned. User {} unauthorised to play.".format(user.id)
        )
        response = HttpResponse("User unauthorized to play", status=401)
    except Avatar.DoesNotExist:
        LOGGER.warning(
            "Avatar does not exist for user {} in game {}".format(user.id, game_id)
        )
        response = HttpResponse("Avatar does not exist for this user", status=404)
    except Http404 as e:
        response = HttpResponse("Game does not exist", status=404)
    except Exception as e:
        LOGGER.error(f"Unknown error occurred while getting connection parameters: {e}")
        response = HttpResponse(
            "Unknown error occurred when getting the current avatar", status=500
        )

    return avatar_id, response



from .spawn_location_finder import SpawnLocationFinder
from .map_updaters import ScoreLocationUpdater, MapContext, PickupUpdater
from .effect_applier import EffectApplier

__all__ = [
    "SpawnLocationFinder",
    "ScoreLocationUpdater",
    "MapContext",
    "PickupUpdater",
    "EffectApplier",
]



"""aimmo autoconfig"""

from .csp_config import CSP_CONFIG

DEFAULT_SETTINGS = {"AUTOCONFIG_INDEX_VIEW": "aimmo/home", "STATIC_URL": "/static/"}

SETTINGS = {
    "INSTALLED_APPS": [
        "common",
        "django.contrib.auth",
        "django.contrib.messages",
        "django.contrib.staticfiles",
        "django_js_reverse",
        "rest_framework",
    ],
    "TEMPLATES": [
        {
            "BACKEND": "django.template.backends.django.DjangoTemplates",
            "APP_DIRS": True,
            "OPTIONS": {
                "context_processors": [
                    "django.template.context_processors.debug",
                    "django.template.context_processors.request",
                    "django.contrib.auth.context_processors.auth",
                    "django.contrib.messages.context_processors.messages",
                ]
            },
        }
    ],
    "USE_TZ": True,
}

SETTINGS.update(CSP_CONFIG)



# Generated by Django 2.2.24 on 2021-07-12 10:50

from django.db import migrations, models

from aimmo.game_creator import generate_game_token
from aimmo.game_manager import GameManager


class Migration(migrations.Migration):

    dependencies = [
        ("aimmo", "0024_unique_class_per_game"),
    ]

    def generate_auth_token_for_games(apps, schema_editor):
        Game = apps.get_model("aimmo", "Game")
        for game in Game.objects.all():
            game.auth_token = generate_game_token()
            game.save()
            game_manager = GameManager()
            game_manager.create_game_secret(game_id=game.id, token=game.auth_token)

    def clear_auth_token_for_games(apps, schema_editor):
        Game = apps.get_model("aimmo", "Game")
        for game in Game.objects.all():
            game.auth_token = ""
            game.save()
            game_manager = GameManager()
            game_manager.delete_game_secret(game_id=game.id)

    operations = [
        migrations.AlterField(
            model_name="game",
            name="auth_token",
            field=models.CharField(max_length=48),
        ),
        migrations.RunPython(
            generate_auth_token_for_games, reverse_code=clear_auth_token_for_games
        ),
    ]



from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from simulation.game_state import GameState
    from simulation.cell import Cell


class TurnState:
    """
    Data structure used by conditions.

    On a given turn, all conditions for a pickup get access to the same TurnState object.
    """

    def __init__(self, game_state: "GameState", interactable_cell: "Cell"):
        self.game_state = game_state
        self.interactable_cell = interactable_cell


def avatar_on_cell(turn_state: TurnState):
    """ Returns an expression that checks if an avatar is on a specified cell. """
    return turn_state.interactable_cell.avatar is not None


def in_backpack(turn_state: TurnState):
    """
    Checks if the interactable is in a backpack.

    The `in_backpack` attribute should be set to True by the `PickupAction`.
    """
    try:
        return turn_state.interactable_cell.interactable.in_backpack
    except Exception:
        return False



from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("aimmo", "0017_add_worksheet_1_and_2"),
    ]

    def set_worksheet_2_as_default_for_games(apps, schema_editor):
        Worksheet = apps.get_model("aimmo", "Worksheet")
        Game = apps.get_model("aimmo", "Game")
        worksheet2 = Worksheet.objects.get(name="Present Day II")
        games_with_no_worksheet = Game.objects.filter(worksheet=None)
        for game in games_with_no_worksheet:
            game.worksheet = worksheet2
            game.save()

    def dummy_reverse(apps, schema_editor):
        """It's not possible to reverse this data migration
        but we want to allow Django to reverse previous migrations.
        """
        pass

    operations = [
        migrations.RunPython(
            set_worksheet_2_as_default_for_games, reverse_code=dummy_reverse
        )
    ]



import random
from logging import getLogger

LOGGER = getLogger(__name__)


class SpawnLocationFinder:
    def __init__(self, world_map):
        self._world_map = world_map

    def potential_spawn_locations(self):
        """
        Used to make sure that the cell is free before spawning.
        """
        return (
            c
            for c in self._world_map.all_cells()
            if c.habitable and not c.avatar and not c.interactable
        )

    def get_random_spawn_locations(self, max_locations):
        if max_locations <= 0:
            return []
        potential_locations = list(self.potential_spawn_locations())
        try:
            return random.sample(potential_locations, max_locations)
        except ValueError:
            return potential_locations

    def get_random_spawn_location(self):
        """Return a single random spawn location.

        Throws:
            IndexError: if there are no possible locations.
        """
        return self.get_random_spawn_locations(1)[0].location



#!/user/bin/env python
import atexit
import os
import platform

import kubernetes
import yaml
from kubernetes.client import AppsV1Api
from kubernetes.config import load_kube_config

from .docker_scripts import build_docker_images
from .shell_api import BASE_DIR, run_command

MINIKUBE_EXECUTABLE = "minikube"


def get_ip():
    internal_ip = str(
        run_command(
            "minikube -p agones ssh grep host.minikube.internal /etc/hosts".split(),
            capture_output=True,
        ).split()[0],
        "utf-8",
    )
    return internal_ip


def create_creator_yaml():
    """
    Loads a replication controller yaml file into a python object.
    """
    orig_path = os.path.join(
        BASE_DIR, "aimmo-game-creator", "aimmo-game-creator-deployment.yaml"
    )
    with open(orig_path) as orig_file:
        content = yaml.safe_load(
            orig_file.read().replace(
                "REPLACE_ME", f"http://{get_ip()}:8000/kurono/api/games/"
            )
        )
    return content


def delete_components():
    apps_api_instance = AppsV1Api()
    for rs in apps_api_instance.list_namespaced_deployment("default").items:
        apps_api_instance.delete_namespaced_deployment(
            body=kubernetes.client.V1DeleteOptions(),
            name=rs.metadata.name,
            namespace="default",
            grace_period_seconds=0,
        )
    delete_fleet_on_exit()


def restart_pods(game_creator_yaml):
    """
    Disables all the components running in the cluster and starts them again
    with fresh updated state.
    :param game_creator_yaml: Replication controller yaml settings file.
    """
    print("Restarting pods")

    run_command(["kubectl", "create", "-f", "agones/fleet.yml"])

    apps_api_instance = AppsV1Api()
    apps_api_instance.create_namespaced_deployment(
        body=game_creator_yaml, namespace="default"
    )


def create_roles():
    """
    Applies the service accounts, roles, and bindings for restricting
    the rights of certain pods and their processses.
    """
    run_command(["kubectl", "apply", "-Rf", "rbac"])


def delete_fleet_on_exit():
    print("Exiting")
    print("Deleting aimmo-game fleet")
    run_command(
        [
            "kubectl",
            "delete",
            "fleet",
            "aimmo-game",
            "--ignore-not-found",
        ]
    )


def start(build_target=None):
    """
    The entry point to the minikube class. Sends calls appropriately to set
    up minikube.
    """
    if platform.machine().lower() not in ("amd64", "x86_64"):
        raise ValueError("Requires 64-bit")
    os.environ["MINIKUBE_PATH"] = MINIKUBE_EXECUTABLE

    # We assume the minikube was started with a profile called "agones"
    load_kube_config(context="agones")

    create_roles()
    build_docker_images(MINIKUBE_EXECUTABLE, build_target=build_target)
    game_creator = create_creator_yaml()
    restart_pods(game_creator)
    atexit.register(delete_components)
    print("Cluster ready")



# -*- coding: utf-8 -*-
# Generated by Django 1.11.24 on 2020-09-29 12:41
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("aimmo", "0019_game_has_worksheet_and_class")]

    operations = [
        migrations.AddField(
            model_name="worksheet",
            name="active_image_path",
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AddField(
            model_name="worksheet",
            name="description",
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name="worksheet",
            name="image_path",
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AddField(
            model_name="worksheet",
            name="short_description",
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name="worksheet",
            name="sort_order",
            field=models.IntegerField(default=0),
        ),
        migrations.AddField(
            model_name="worksheet",
            name="thumbnail_image_path",
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AddField(
            model_name="worksheet",
            name="thumbnail_text",
            field=models.CharField(blank=True, max_length=100),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.24 on 2020-03-02 13:10
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("aimmo", "0011_reset_game_tokens"),
    ]

    operations = [
        migrations.AlterField(
            model_name="game",
            name="generator",
            field=models.CharField(
                choices=[("Main", "Open World"), ("Level1", "Level 1")],
                default="Main",
                max_length=20,
            ),
        ),
        migrations.AlterField(
            model_name="game",
            name="status",
            field=models.CharField(
                choices=[("r", "running"), ("s", "stopped"), ("p", "paused")],
                default="r",
                max_length=1,
            ),
        ),
    ]



from django.db import migrations


class Migration(migrations.Migration):
    def add_worksheet_1_and_2(apps, schema_editor):
        starter_code = """def next_turn(world_state, avatar_state):
    new_dir = direction.NORTH
    # Your code goes here
    action = MoveAction(new_dir)
    return action
"""
        Worksheet = apps.get_model("aimmo", "Worksheet")
        worksheet1 = Worksheet.objects.create(
            name="Present Day I: The Museum", starter_code=starter_code
        )
        worksheet2 = Worksheet.objects.create(
            name="Present Day II", starter_code=starter_code
        )
        worksheet1.save()
        worksheet2.save()

    def remove_worksheets(apps, schema_editor):
        Worksheet = apps.get_model("aimmo", "Worksheet")
        Worksheet.objects.all().delete()

    dependencies = [
        ("aimmo", "0016_game_class"),
    ]

    operations = [
        migrations.RunPython(add_worksheet_1_and_2, reverse_code=remove_worksheets)
    ]



# Generated by Django 2.0 on 2020-10-28 17:30

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("aimmo", "0022_allow_games_to_have_no_name"),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="levelattempt",
            unique_together=set(),
        ),
        migrations.RemoveField(
            model_name="levelattempt",
            name="game",
        ),
        migrations.RemoveField(
            model_name="levelattempt",
            name="user",
        ),
        migrations.DeleteModel(
            name="LevelAttempt",
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.24 on 2020-06-29 10:17
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("aimmo", "0013_alter_game_can_play"),
    ]

    operations = [
        migrations.CreateModel(
            name="Worksheet",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=100)),
                (
                    "era",
                    models.PositiveSmallIntegerField(
                        choices=[
                            (1, "future"),
                            (2, "ancient"),
                            (3, "modern day"),
                            (4, "prehistoric"),
                            (5, "broken future"),
                        ],
                        default=1,
                    ),
                ),
                ("starter_code", models.TextField()),
            ],
        ),
    ]




import secrets
from aimmo.avatar_creator import create_avatar_for_user
from aimmo.game_manager import GameManager

NUM_BYTES_FOR_TOKEN_GENERATOR = 32
TOKEN_MAX_LENGTH = 48


def generate_game_token():
    return secrets.token_urlsafe(nbytes=NUM_BYTES_FOR_TOKEN_GENERATOR)[
        :TOKEN_MAX_LENGTH
    ]


def create_game(main_user, form):
    """
    Creates a Game by:
    - saving the form
    - setting default values
    - adding users who can play the game
    - creating an avatar for the main user.
    - creating the game secret in game manager
    :param main_user: The user who requested game creation, and is the game owner.
    :param form: The form used to submit the creation of the game.
    :param users_to_add_to_game: List of User objects who are able to play this game.
    :return: The initialised Game object.
    """
    game = form.save(commit=False)
    game.auth_token = generate_game_token()
    game.generator = "Main"
    game.owner = main_user
    game.main_user = main_user
    game.save()
    create_avatar_for_user(main_user, game.id)
    game_manager = GameManager()
    game_manager.create_game_secret(game_id=game.id, token=game.auth_token)
    return game



from __future__ import annotations

from typing import Dict

from simulation.avatar.avatar_wrapper import AvatarWrapper


def worksheet1_avatar_state_serializer(avatar: AvatarWrapper) -> Dict:
    return {
        "location": avatar.location.serialize(),
        "id": avatar.player_id,
        "orientation": avatar.orientation,
    }


def worksheet2_avatar_state_serializer(avatar: AvatarWrapper) -> Dict:
    return {
        "location": avatar.location.serialize(),
        "id": avatar.player_id,
        "orientation": avatar.orientation,
        "backpack": [artefact.serialize() for artefact in avatar.backpack],
    }


def worksheet3_avatar_state_serializer(avatar: AvatarWrapper) -> Dict:
    return {
        "location": avatar.location.serialize(),
        "id": avatar.player_id,
        "orientation": avatar.orientation,
        "backpack": [artefact.serialize() for artefact in avatar.backpack],
    }



# Generated by Django 2.2.24 on 2021-11-18 11:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("aimmo", "0025_generate_auth_token"),
    ]

    operations = [
        migrations.AlterField(
            model_name="game",
            name="auth_token",
            field=models.CharField(blank=True, max_length=48),
        ),  # Generated by Django, but doesn't actually change anything
        migrations.AlterField(
            model_name="game",
            name="worksheet",
            field=models.IntegerField(default=1),
        ),
        migrations.DeleteModel(
            name="Worksheet",
        ),
        migrations.RenameField(
            model_name="game",
            old_name="worksheet",
            new_name="worksheet_id",
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [migrations.swappable_dependency(settings.AUTH_USER_MODEL)]

    operations = [
        migrations.CreateModel(
            name="Avatar",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                (
                    "player",
                    models.ForeignKey(
                        to=settings.AUTH_USER_MODEL, on_delete=models.CASCADE
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Player",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                ("code", models.TextField()),
                (
                    "user",
                    models.OneToOneField(
                        to=settings.AUTH_USER_MODEL, on_delete=models.CASCADE
                    ),
                ),
            ],
        ),
    ]



from dataclasses import dataclass
from typing import List


@dataclass
class Worksheet:
    id: int
    name: str
    era: int
    starter_code: str
    active_image_path: str
    description: str
    image_path: str
    short_description: str
    sort_order: int
    thumbnail_image_path: str
    thumbnail_text: str
    student_challenge_url: str

    def __post_init__(self):
        # Remove spaces around starter_code and add a new line
        self.starter_code = self.starter_code.strip() + "\n"


WORKSHEETS = {
    1: Worksheet(
        id=1,
        name="Present Day I: The Museum",
        era=1,
        starter_code="""
#-------------------------------------------------------------------------------
#  Worksheet 1 challenges:
#    Task 1: Change direction
#       - The starter code moves your avatar North. Change this code so that it
#         moves in a different direction.
#
#    Task 2: Move in all directions
#       - Write some code that moves your avatar in a random direction. Don't
#         forget to add `import random` at the top of your code.
#
#    Task 3: Investigate location
#       - Investigate what `print(avatar_state.location)` tells you and then
#         check if you can move to the next location
#
#  New commands:
#       - MoveAction(DIRECTION)
#       - avatar_state.location -> LOCATION
#       - world_state.can_move_to(LOCATION) -> True/False
#
#-------------------------------------------------------------------------------
def next_turn(world_state, avatar_state):
    new_dir = direction.NORTH
    #  Your code goes here
    action = MoveAction(new_dir)
    return action
""",
        active_image_path="images/worksheets/future_active.png",
        description="While working at the Scriptsonian museum for class credit, you have accidentally activated the mysterious Kurono time machine! The museum’s valuable artefacts are scattered across the shelves, the floor, the whole spacetime continuum. It’s going to take more than just a backpack to collect them all. You need to use the Kurono time machine to get them back – but before jumping in you need to learn how to use its navigation system. This challenge shows you how to move around the map so you don’t run into problems in the future…or past.",
        image_path="images/worksheets/future.jpg",
        short_description="While working at the Scriptsonian museum for class credit, you have accidentally activated the mysterious Kurono time machine!",
        sort_order=100,
        thumbnail_image_path="images/worksheets/lock.png",
        thumbnail_text="",
        student_challenge_url="https://code-for-life.gitbook.io/learning-resources/student-challenges/student-challenge-1",
    ),
    2: Worksheet(
        id=2,
        name="Present Day II",
        era=1,
        starter_code="""
#-------------------------------------------------------------------------------
#  Worksheet 2 challenges:
#    Task 1: Pick up an artefact
#       - Pick up any artefacts you find on the board. If the location you are
#         in contains an artefact, you can pick it up by returning a
#         PickupAction() instead of a MoveAction(DIRECTION).
#
#  New commands:
#       - action = PickupAction()
#
#  Previous commands:
#       - avatar_state.location -> LOCATION
#       - MoveAction(DIRECTION)
#       - world_state.can_move_to(LOCATION) -> True/False
#-------------------------------------------------------------------------------
import random

def next_turn(world_state, avatar_state):
    #  Choose a random direction to move in
    number = random.randint(1,4)
    if number == 1:
        new_dir = direction.NORTH
    elif number == 2:
        new_dir = direction.EAST
    elif number == 3:
        new_dir = direction.SOUTH
    else:
        new_dir = direction.WEST
    #  Get details about the location you would move onto
    next_location = avatar_state.location + new_dir
    if world_state.can_move_to(next_location):
        #  This location is free
        action = MoveAction(new_dir)
    else:
        print("I can't move that way!")
        action = WaitAction()
    return action
""",
        active_image_path="images/worksheets/future2_active.png",
        description="Now that you know how to navigate, it’s time to clean up the evidence before leaving this timeline!  Some of the artefacts escaped the vortex and ended up all over the museum - if each of you picks one of them up, you should be ready to embark on your journey!",
        image_path="images/worksheets/future.jpg",
        short_description="Now that you know how to navigate, it’s time to clean up the evidence before leaving this timeline!",
        sort_order=200,
        thumbnail_image_path="images/worksheets/lock.png",
        thumbnail_text="",
        student_challenge_url="https://code-for-life.gitbook.io/learning-resources/student-challenges/student-challenge-2",
    ),
    3: Worksheet(
        id=3,
        name="Ancient",
        era=2,
        starter_code="""
#-------------------------------------------------------------------------------
#  Worksheet 3 challenges:
#     Task 1: Direction control
#       - Return MoveTowardsAction(ARTEFACT) to move towards the nearest
#         artefact.
#       - You can use world_state.scan_nearby() to get a list of nearby
#         artefacts.
#       - You must check that the list is not empty before reading a value from
#         it.
#
#     Task 2: Pick up five artefacts
#       - Pick up five artefacts on the board. Check your backpack to see how
#         many you are holding using `avatar_state.backpack`. Output a message
#         when you have picked up five.
#
#     Task 3: Types of artefacts
#       - Part 1: Use `artefact.type` where `artefact` is an item of
#         `avatar_state.backpack` to list all types of artefacts in your
#         backpack.
#       - Part 2: Output a summary of your backpack using the same
#         `artefact.type` as above. e.g. I have 3 keys and 2 chests.
#
#  New commands:
#       - world_state.scan_nearby() -> LIST OF ARTEFACTS
#       - MoveTowardsAction(ARTEFACT)
#       - avatar_state.backpack -> LIST OF HELD ARTEFACTS
#       - artefact.type -> TYPE (e.g. artefact_types.KEY or artefact_types.CHEST)
#
#  Previous commands:
#       - avatar_state.location -> LOCATION
#       - MoveAction(DIRECTION)
#       - world_state.can_move_to(LOCATION) -> True/False
#       - action = PickupAction()
#-------------------------------------------------------------------------------
def next_turn(world_state, avatar_state):
    #  scan_nearby() returns a list of the nearest artefacts
    nearby = world_state.scan_nearby(avatar_state.location)

    #  Head towards the nearest artefact
    nearest = nearby[0]
    action = MoveTowardsAction(nearest)

    return action
""",
        active_image_path="images/worksheets/ancient_active.png",
        description="You are now familiar with your equipment and have managed to unlock the gateway to the first time checkpoint. You find yourself in a strange place, full of familiar structures in the form of ruins. You see artefacts of different types scattered around you, where to start? Luckily, your navigator is working... You can use it to scan your surroundings and walk towards the artefacts. If you and your friends pick up five artefacts each, you should be done in no time!",
        image_path="images/worksheets/ancient.jpg",
        short_description="You find yourself in a strange place, full of familiar structures in the form of ruins.",
        sort_order=300,
        thumbnail_image_path="images/worksheets/lock.png",
        thumbnail_text="",
        student_challenge_url="https://code-for-life.gitbook.io/learning-resources/student-challenges/student-challenge-3",
    ),
    4: Worksheet(
        id=4,
        name="21st Century",
        era=3,
        starter_code="""
def next_turn(world_state, avatar_state):
    new_dir = direction.NORTH
    # Your code goes here
    action = MoveAction(new_dir)
    return action
""",
        active_image_path="images/worksheets/modern_active.png",
        description="After successfully collecting all the missing artefacts from the first time checkpoint, you arrive at what looks like the 21st century. You recognise some cars parked here and there, old-fashioned roads and houses like the ones your history teacher told you about. On the bright side, you seem to be alone and safe to walk around... for now. A more recent timeline doesn’t make artefacts easier to find, though. Or at least not the right ones. In this timeline there seems to be an amount of falsified objects that resemble the ones you’re looking for, but aren’t quite genuine. These will have no value in the museum. Your navigation system will be able to tell you whether an object is genuine or not, but it’s up to you to decide which ones to bring back!",
        image_path="images/worksheets/modern.jpg",
        short_description="After successfully collecting all the missing artefacts from the first time checkpoint, you arrive at what looks like the 21st century.",
        sort_order=400,
        thumbnail_image_path="",
        thumbnail_text="Coming Soon",
        student_challenge_url="",
    ),
    5: Worksheet(
        id=5,
        name="Prehistoric",
        era=4,
        starter_code="""
def next_turn(world_state, avatar_state):
    new_dir = direction.NORTH
    # Your code goes here
    action = MoveAction(new_dir)
    return action
""",
        active_image_path="images/worksheets/prehistory_active.png",
        description="Oh no! Looks like the time machine has taken you even further back in time. You find yourself in a place that looks very… wild. You can hear the occasional distant roaring, added to the buzzing of what sound like  gigantic  insects. Vegetation is so abundant around you that you can barely move. It might be a good idea to use your navigation system to scan all around you before even trying! And don’t forget to keep track of the portal that’ll send you back to your original timeline - just in case something with sharp teeth starts running towards you.",
        image_path="images/worksheets/prehistory.jpg",
        short_description="Oh no! Looks like the time machine has taken you even further back in time. You find yourself in a place that looks very… wild.",
        sort_order=500,
        thumbnail_image_path="",
        thumbnail_text="Coming Soon",
        student_challenge_url="",
    ),
    6: Worksheet(
        id=6,
        name="Back to the Future",
        era=5,
        starter_code="""
def next_turn(world_state, avatar_state):
    new_dir = direction.NORTH
    # Your code goes here
    action = MoveAction(new_dir)
    return action
""",
        active_image_path="images/worksheets/broken_future_active.png",
        description="You’ve made it! Well, almost. You’re back to your original time, but something seems to be off. The museum is still there, with all the chaos you still need to tidy up, but the environment isn’t quite right. The electronic containers and shelves seem slightly different, and when the AI interlocutor speaks, you don’t recognise the language. It looks like you’ve changed something in the past that affected events in your present! If you manage to extract the metadata from the recovered artefacts, you should be able to find out which timelines they were found in. You can input this text into the Kurono machine to get things back to normal.",
        image_path="images/worksheets/broken_future.jpg",
        short_description="You’ve made it! Well, almost. You’re back to your original time, but something seems to be off.",
        sort_order=600,
        thumbnail_image_path="",
        thumbnail_text="Coming Soon",
        student_challenge_url="",
    ),
}


def get_complete_worksheets() -> List[Worksheet]:
    return [
        worksheet
        for worksheet in WORKSHEETS.values()
        if worksheet.thumbnail_text != "Coming Soon"
    ]


def get_incomplete_worksheets() -> List[Worksheet]:
    return [
        worksheet
        for worksheet in WORKSHEETS.values()
        if worksheet.thumbnail_text == "Coming Soon"
    ]


def get_worksheets_excluding_id(id: int) -> List[Worksheet]:
    return [worksheet for worksheet in WORKSHEETS.values() if worksheet.id != id]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2019-04-04 14:55
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("aimmo", "0008_default_public_field")]

    operations = [
        migrations.AddField(
            model_name="game",
            name="status",
            field=models.CharField(
                choices=[(b"r", b"running"), (b"s", b"stopped"), (b"p", b"paused")],
                default=b"r",
                max_length=1,
            ),
        )
    ]
